@@ -73,7 +73,6 @@ G_MODULE_EXPORT void about_action_callback (GtkAction *action, Totem *totem);
 G_MODULE_EXPORT void plugins_action_callback (GtkAction *action, Totem *totem);
 G_MODULE_EXPORT void repeat_mode_action_callback (GtkToggleAction *action, Totem *totem);
 G_MODULE_EXPORT void shuffle_mode_action_callback (GtkToggleAction *action, Totem *totem);
-G_MODULE_EXPORT void deinterlace_action_callback (GtkToggleAction *action, Totem *totem);
 G_MODULE_EXPORT void show_controls_action_callback (GtkToggleAction *action, Totem *totem);
 G_MODULE_EXPORT void show_sidebar_action_callback (GtkToggleAction *action, Totem *totem);
 G_MODULE_EXPORT void aspect_ratio_changed_callback (GtkRadioAction *action, GtkRadioAction *current, Totem *totem);
@@ -189,10 +188,12 @@ create_lang_actions (Totem *totem, GtkActionGroup *action_group, guint ui_id,
 
 	if (is_lang == FALSE) {
 		add_lang_action (totem, action_group, ui_id, paths, prefix,
+		                /* Translators: an entry in the "Languages" menu, used to choose the audio language of a DVD */
 				_("None"), -2, 0, &group);
 	}
 
 	action = add_lang_action (totem, action_group, ui_id, paths, prefix,
+	               /* Translators: an entry in the "Languages" menu, used to choose the audio language of a DVD */
 			_("Auto"), -1, 0, &group);
 
 	i = 0;
@@ -366,10 +367,6 @@ connect_proxy_cb (GtkActionGroup *action_group,
                   gpointer data)
 {
         GtkLabel *label;
-        GtkRecentInfo *recent_info;
-        GdkPixbuf *icon;
-        GtkWidget *image = NULL;
-        gint w, h;
 
         if (!GTK_IS_MENU_ITEM (proxy))
                 return;
@@ -378,23 +375,6 @@ connect_proxy_cb (GtkActionGroup *action_group,
 
         gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
         gtk_label_set_max_width_chars (label,TOTEM_MAX_RECENT_ITEM_LEN);
-
-        if (!GTK_IS_IMAGE_MENU_ITEM (proxy))
-                return;
-
-        recent_info = g_object_get_data (G_OBJECT (action), "recent-info");
-        g_assert (recent_info != NULL);
-
-        gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-
-        icon = gtk_recent_info_get_icon (recent_info, w);
-        if (icon != NULL) {
-                image = gtk_image_new_from_pixbuf (icon);
-                g_object_unref (icon);
-        }
-
-        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (proxy), image);
-        gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (proxy), TRUE);
 }
 
 static void
@@ -484,6 +464,9 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
                 const char    *display_name;
                 char          *label;
                 char          *escaped_label;
+                const gchar   *mime_type;
+                gchar         *content_type;
+                GIcon         *icon = NULL;
 
                 info = (GtkRecentInfo *) l->data;
 
@@ -506,6 +489,18 @@ totem_recent_manager_changed_callback (GtkRecentManager *recent_manager, Totem *
                                   G_CALLBACK (on_recent_file_item_activated),
                                   totem);
 
+                mime_type = gtk_recent_info_get_mime_type (info);
+                content_type = g_content_type_from_mime_type (mime_type);
+                if (content_type != NULL) {
+                        icon = g_content_type_get_icon (content_type);
+                        g_free (content_type);
+                }
+                if (icon != NULL) {
+                        gtk_action_set_gicon (action, icon);
+                        gtk_action_set_always_show_image (action, TRUE);
+                        g_object_unref (icon);
+                }
+
                 gtk_action_group_add_action (totem->recent_action_group,
                                             action);
                 g_object_unref (action);
@@ -1126,13 +1121,13 @@ previous_chapter_action_callback (GtkAction *action, Totem *totem)
 void
 skip_forward_action_callback (GtkAction *action, Totem *totem)
 {
-	totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000);
+	totem_action_seek_relative (totem, SEEK_FORWARD_OFFSET * 1000, FALSE);
 }
 
 void
 skip_backwards_action_callback (GtkAction *action, Totem *totem)
 {
-	totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000);
+	totem_action_seek_relative (totem, SEEK_BACKWARD_OFFSET * 1000, FALSE);
 }
 
 void
@@ -1264,17 +1259,6 @@ shuffle_mode_action_callback (GtkToggleAction *action, Totem *totem)
 			gtk_toggle_action_get_active (action));
 }
 
-void
-deinterlace_action_callback (GtkToggleAction *action, Totem *totem)
-{
-	gboolean value;
-
-	value = gtk_toggle_action_get_active (action);
-	bacon_video_widget_set_deinterlacing (totem->bvw, value);
-	gconf_client_set_bool (totem->gc, GCONF_PREFIX"/deinterlace",
-			value, NULL);
-}
-
 void
 show_controls_action_callback (GtkToggleAction *action, Totem *totem)
 {
